/*
 *
 * Copyright (c) 2003-2004 Erez Zadok
 * Copyright (c) 2003-2004 Swapnil V. Patil
 * Copyright (c) 2003-2004 Anand Kashyap
 * Copyright (c) 2003-2004 Gopalan Sivathanu
 * Copyright (c) 2003-2004 Stony Brook University
 * Copyright (c) 2003-2004 The Research Foundation of State University of New York
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package.
 *
 * This Copyright notice must be kept intact and distributed with all sources.
 */

/*
 *  $Id: super.c,v 1.17 2004/08/01 21:26:16 gopalan Exp $
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */
#ifdef FISTGEN
# include "fist_i3fs.h"
#endif /* FISTGEN */
#include "fist.h"
#include "i3fs.h"
#include "i3fs_utils.h"


STATIC void
i3fs_close_databases(super_block_t *sb)	{
	DB *policydb, *datadb, *metadb, *accessdb;
	policydb = stopd(sb)->policydb;
	datadb = stopd(sb)->datadb;
	metadb = stopd(sb)->metadb;
	accessdb = stopd(sb)->accessdb;


	policydb->sync(policydb, 0);
        datadb->sync(datadb, 0);
        metadb->sync(metadb, 0);
        accessdb->sync(accessdb, 0);

	policydb->close(policydb, 0);
	datadb->close(datadb, 0);
	metadb->close(metadb, 0);
	accessdb->close(accessdb, 0);
}


STATIC void
i3fs_read_inode(inode_t *inode)
{
    int i;

    print_entry_location();

    itopd(inode) = KMALLOC(sizeof(struct i3fs_inode_info), GFP_KERNEL);
  
    // initialize values for the integrity checking fields.

 
    itopd(inode)->policy = 0;
    itopd(inode)->check_result = 0;
    itopd(inode)->whole_file_result = 0;
    itopd(inode)->check_required = 1;


    for (i=0;i<10;i++)
	itopd(inode)->page_bitmap[i] = 0;


	
    if (!itopd(inode)) {
	printk("<0>%s:%s:%d: No kernel memory!\n", __FILE__, __FUNCTION__, __LINE__);
	ASSERT(NULL);
    }
    itohi(inode) = NULL;

    inode->i_version = ++event;	/* increment inode version */
    inode->i_op = &i3fs_main_iops;
    inode->i_fop = &i3fs_main_fops;
#if 0
    /*
     * XXX: To export a file system via NFS, it has to have the
     * FS_REQUIRES_DEV flag, so turn it on.  But should we inherit it from
     * the lower file system, or can we allow our file system to be exported
     * even if the lower one cannot be natively exported.
     */
    inode->i_sb->s_type->fs_flags |= FS_REQUIRES_DEV;
    /*
     * OK, the above was a hack, which is now turned off because it may
     * cause a panic/oops on some systems.  The correct way to export a
     * "nodev" filesystem is via using nfs-utils > 1.0 and the "fsid=" export
     * parameter, which requires 2.4.20 or later.
     */
#endif
    inode->i_mapping->a_ops = &i3fs_aops;

    print_exit_location();
}


#if defined(FIST_DEBUG) || defined(FIST_FILTER_SCA)
/*
 * No need to call write_inode() on the lower inode, as it
 * will have been marked 'dirty' anyway. But we might need
 * to write some of our own stuff to disk.
 */
STATIC void
i3fs_write_inode(inode_t *inode, int sync)
{
    print_entry_location();

    itopd(inode)->check_result = 0;

/*    if (stopd(inode->i_sb)->allow_updates)	{
	printk("\nUpdating inode checksum");
	if (itopd(inode)->policy != 0)	{
		remove_from_db(stopd(inode->i_sb)->metadb, &(itohi(inode)->i_ino), sizeof(long int));
		checksum_metadata(inode, itopd(inode)->policy);
	        itopd(inode)->check_result = 0;
	}
	
    }*/

    itopd(inode)->check_result = 0;
    print_exit_location();
}
#endif /* defined(FIST_DEBUG) || defined(FIST_FILTER_SCA) */


STATIC void
i3fs_put_inode(inode_t *inode)
{
    print_entry_location();
    fist_dprint(8, "%s i_count = %d, i_nlink = %d\n", __FUNCTION__,
		atomic_read(&inode->i_count), inode->i_nlink);

    itopd(inode)->check_result = 0;

    if (stopd(inode->i_sb)->allow_updates)      {
        if (itopd(inode)->policy != 0)  {
     //           remove_from_db(stopd(inode->i_sb)->metadb, &(itohi(inode)->i_ino), sizeof(long int));
                checksum_metadata(inode, itopd(inode)->policy);
                itopd(inode)->check_result = 0;
        }

    }

    /*
     * This is really funky stuff:
     * Basically, if i_count == 1, iput will then decrement it and this inode will be destroyed.
     * It is currently holding a reference to the hidden inode.
     * Therefore, it needs to release that reference by calling iput on the hidden inode.
     * iput() _will_ do it for us (by calling our clear_inode), but _only_ if i_nlink == 0.
     * The problem is, NFS keeps i_nlink == 1 for silly_rename'd files.
     * So we must for our i_nlink to 0 here to trick iput() into calling our clear_inode.
     */
    if (atomic_read(&inode->i_count) == 1)
	inode->i_nlink = 0;
    print_exit_location();
}


#if defined(FIST_DEBUG) || defined(FIST_FILTER_SCA)
/*
 * we now define delete_inode, because there are two VFS paths that may
 * destroy an inode: one of them calls clear inode before doing everything
 * else that's needed, and the other is fine.  This way we truncate the inode
 * size (and its pages) and then clear our own inode, which will do an iput
 * on our and the lower inode.
 */
STATIC void
i3fs_delete_inode(inode_t *inode)
{
    print_entry_location();

    fist_checkinode(inode, "i3fs_delete_inode IN");
    inode->i_size = 0;		/* every f/s seems to do that */
    clear_inode(inode);

    print_exit_location();
}
#endif /* defined(FIST_DEBUG) || defined(FIST_FILTER_SCA) */


/* final actions when unmounting a file system */
STATIC void
i3fs_put_super(super_block_t *sb)
{
    print_entry_location();

    if (stopd(sb)) {
	mntput(stopd(sb)->hidden_mnt);
        i3fs_close_databases(sb);
	KFREE(stopd(sb));
	stopd(sb) = NULL;
    }

    fist_dprint(6, "i3fs: released super\n");


/* Freeing up the global variables on unmount */
    print_exit_location();
}


#ifdef NOT_NEEDED
/*
 * This is called in do_umount before put_super.
 * The superblock lock is not held yet.
 * We probably do not need to define this or call write_super
 * on the hidden_sb, because sync_supers() will get to hidden_sb
 * sooner or later.  But it is also called from file_fsync()...
 */
STATIC void
i3fs_write_super(super_block_t *sb)
{
    return;
}
#endif /* NOT_NEEDED */


STATIC int
i3fs_statfs(super_block_t *sb, struct statfs *buf)
{
    int err = 0;
    super_block_t *hidden_sb;

    print_entry_location();
    hidden_sb = stohs(sb);
    err = vfs_statfs(hidden_sb, buf);

    print_exit_status(err);
    return err;
}


/*
 * XXX: not implemented.  This is not allowed yet.
 * Should we call this on the hidden_sb?  Probably not.
 */
STATIC int
i3fs_remount_fs(super_block_t *sb, int *flags, char *data)
{
    return -ENOSYS;
}


/*
 * Called by iput() when the inode reference count reached zero
 * and the inode is not hashed anywhere.  Used to clear anything
 * that needs to be, before the inode is completely destroyed and put
 * on the inode free list.
 */
STATIC void
i3fs_clear_inode(inode_t *inode)
{

    print_entry_location();

    fist_checkinode(inode, "i3fs_clear_inode IN");
    /*
     * Decrement a reference to a hidden_inode, which was incremented
     * by our read_inode when it was created initially.
     */

    itopd(inode)->check_result = 0;
    iput(itohi(inode));
    // XXX: why this assertion fails?
    // because it doesn't like us
    // ASSERT((inode->i_state & I_DIRTY) == 0);
    KFREE(itopd(inode));
    itopd(inode) = NULL;

    print_exit_location();
}


/*
 * Called in do_umount() if the MNT_FORCE flag was used and this
 * function is defined.  See comment in linux/fs/super.c:do_umount().
 * Used only in nfs, to kill any pending RPC tasks, so that subsequent
 * code can actually succeed and won't leave tasks that need handling.
 *
 * PS. I wonder if this is somehow useful to undo damage that was
 * left in the kernel after a user level file server (such as amd)
 * dies.
 */
STATIC void
i3fs_umount_begin(super_block_t *sb)
{
    super_block_t *hidden_sb;

    print_entry_location();

    hidden_sb = stohs(sb);

    if (hidden_sb->s_op->umount_begin)
	hidden_sb->s_op->umount_begin(hidden_sb);
    

    print_exit_location();
}

/* Called to print options in /proc/mounts */
static int i3fs_show_options(struct seq_file *m, struct vfsmount *mnt) {
	struct super_block *sb = mnt->mnt_sb;
	int ret = 0;
	unsigned long tmp = 0;
	char *path;

	tmp = __get_free_page(GFP_KERNEL);
	if (!tmp) {
		ret = -ENOMEM;
		goto out;
	}

	path = d_path(dtohd(sb->s_root), stopd(sb)->hidden_mnt, (char *)tmp, PAGE_SIZE);

	seq_printf(m, ",dir=%s", path);
	seq_printf(m, ",debug=%d", fist_get_debug_value());

out:
	if (tmp) {
		free_page(tmp);
	}
	return ret;
}


struct super_operations i3fs_sops =
{
    read_inode:		i3fs_read_inode,
#if defined(FIST_DEBUG) || defined(FIST_FILTER_SCA)
    write_inode:	i3fs_write_inode,
#endif /* defined(FIST_DEBUG) || defined(FIST_FILTER_SCA) */
    put_inode:		i3fs_put_inode,
#if defined(FIST_DEBUG) || defined(FIST_FILTER_SCA)
    delete_inode:	i3fs_delete_inode,
#endif /* defined(FIST_DEBUG) || defined(FIST_FILTER_SCA) */
    put_super:		i3fs_put_super,
    statfs:		i3fs_statfs,
    remount_fs:		i3fs_remount_fs,
    clear_inode:	i3fs_clear_inode,
    umount_begin:	i3fs_umount_begin,
    show_options:	i3fs_show_options,
};

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
